@@ -16,6 +16,7 @@ from six.moves.urllib import parse
 from . import constants
 from . import utils
 from .exceptions import (
+    CannotInferTicketNumberFromFolderName,
     GitCommandError,
     LocalCopyOutOfDate,
     NotTicketFolderException
@@ -134,29 +135,135 @@ def debug(args, jira, path, **kwargs):
     ipdb.set_trace()
 
 
+@command('List which plugins are currently enabled')
+def plugins(args, jira, path, **kwargs):
+    def build_plugin_dict(enabled, available):
+        all_plugins = {}
+        for plugin_name, cls in available.items():
+            all_plugins[plugin_name] = {
+                'enabled': False,
+                'class': cls,
+            }
+        for plugin_instance in enabled:
+            plugin_name = plugin_instance.plugin_name
+            all_plugins[plugin_name]['enabled'] = True
+            all_plugins[plugin_name]['instance'] = plugin_instance
+
+        return all_plugins
+
+    t = Terminal()
+    parser = argparse.ArgumentParser()
+    parser.add_argument('--verbose', action='store_true', default=False)
+    parser.add_argument(
+        '--enabled-only',
+        dest='enabled_only',
+        action='store_true',
+        default=False
+    )
+    parser.add_argument(
+        '--disabled-only',
+        dest='disabled_only',
+        action='store_true',
+        default=False
+    )
+    parser.add_argument(
+        '--enable',
+    )
+    parser.add_argument(
+        '--disable',
+    )
+    parser.add_argument(
+        '--global',
+        dest='set_global',
+        default=False,
+        action='store_true',
+    )
+    args = parser.parse_args(args)
+
+    if args.disabled_only and args.enabled_only:
+        parser.error(
+            "--disabled-only and --enabled-only are mutually exclusive."
+        )
+
+    folder = TicketFolder(path, jira, quiet=True)
+    enabled_plugins = folder.load_plugins()
+    available_plugins = utils.get_installed_plugins()
+
+    if args.enable:
+        if args.enable not in available_plugins:
+            parser.error(
+                "Plugin '%s' is not installed." % args.enable
+            )
+        if args.set_global:
+            utils.set_global_config_value(
+                'plugins', args.enable, 'enabled',
+            )
+        else:
+            folder.set_config_value(
+                'plugins', args.enable, 'enabled'
+            )
+    elif args.disable:
+        if args.set_global:
+            utils.set_global_config_value(
+                'plugins', args.disable, 'disabled',
+            )
+        else:
+            folder.set_config_value(
+                'plugins', args.disable, 'disabled'
+            )
+    else:
+        all_plugins = build_plugin_dict(enabled_plugins, available_plugins)
+
+        for plugin_name, plugin_data in all_plugins.items():
+            if plugin_data['enabled'] and args.disabled_only:
+                continue
+            if not plugin_data['enabled'] and args.enabled_only:
+                continue
+            if plugin_data['enabled']:
+                color = t.bold
+            else:
+                color = t.normal
+
+            print(
+                color + plugin_name + t.normal
+                + (' (Enabled)'
+                    if plugin_data['enabled']
+                    else (
+                        ' (Disabled; enable by running `jirafs '
+                        'plugins --enable=%s`)' % plugin_name
+                    )
+                )
+            )
+            if args.verbose:
+                for line in plugin_data['class'].__doc__.strip().split('\n'):
+                    print('     %s' % line)
+
+
 @command('Get the status of the current folder', try_subfolders=True)
 def status(args, jira, path, **kwargs):
     t = Terminal()
 
-    def format_field_changes(changes, color):
+    def format_field_changes(changes, color, no_upload=False):
         lines = []
         color = getattr(t, color)
         normal = t.normal
 
-        for filename in changes['files']:
+        for filename in changes.get('files', []):
             lines.append(
-                '\t' + color + filename + normal + ' (file upload)'
+                '\t' + color + filename + normal + (
+                    ' (save to repo)' if no_upload else ' (file upload)'
+                )
             )
-        for field, value_set in changes['fields'].items():
+        for field, value_set in changes.get('fields', {}).items():
             lines.append(
                 '\t' + color + field + normal +
                 ' (field changed from \'%s\' to \'%s\')' % value_set
             )
-        if changes['new_comment']:
+        if changes.get('new_comment', ''):
             lines.append(
                 '\t' + color + '[New Comment]' + normal
             )
-            for line in changes['new_comment'].split('\n'):
+            for line in changes.get('new_comment', '').split('\n'):
                 lines.append(
                     '\t\t' + line
                 )
@@ -208,6 +315,24 @@ def status(args, jira, path, **kwargs):
                 format_field_changes(staged, 'red')
             )
 
+        local_uncommitted = folder_status['local_uncommitted']
+        if local_uncommitted['files']:
+            printed_changes = True
+            print('')
+            print(
+                "Uncommitted changes prevented from being sent to JIRA "
+                "because they match at least one of the patterns in your "
+                ".jirafs_ignore file; use `jirafs commit` to commit these "
+                "changes."
+            )
+            print(
+                "Note: these files will " + t.bold + "not" + t.normal + " "
+                "be uploaded to JIRA even after being committed."
+            )
+            print(
+                format_field_changes(local_uncommitted, 'cyan', no_upload=True)
+            )
+
         if not printed_changes:
             print('No changes found')
         else:
@@ -335,14 +460,7 @@ def config(args, jira, path, **kwargs):
         value = args.params[1]
 
         if args.global_config:
-            config = utils.get_config()
-            if not config.has_section(section):
-                config.add_section(section)
-            config.set(section, key, value)
-            with open(
-                os.path.expanduser('~/%s' % constants.GLOBAL_CONFIG), 'w'
-            ) as out:
-                config.write(out)
+            utils.set_global_config_value(section, key, value)
         else:
             try:
                 folder = TicketFolder(path, jira)
@@ -401,7 +519,7 @@ def main():
             print("    %s" % line)
         print("")
         sys.exit(1)
-    except NotTicketFolderException:
+    except (NotTicketFolderException, CannotInferTicketNumberFromFolderName):
         if not fn.try_subfolders:
             print(
                 "The command '%s' must be ran from within an issue folder." % (
